@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-import sys, re, os, uuid
+import sys, re, os, uuid, hashlib
 
 from datetime import *
 
@@ -9,34 +9,24 @@ from Perdy.pretty import *
 
 
 #________________________________________________________________
-def generateID():
+def generateID(scope=None, name=None):
 	"""
 	todo: cache uuid based on package path Namespac
 	"""
-	return 'EDDO_%s' % uuid.uuid4()
+	if name:
+		m = hashlib.md5()
+		m.update(f'{scope}_{name}'.encode('utf8'))
+		return 'EDDO_%s'%m.hexdigest().upper()
+
+	return 'EDDO_%s'%uuid.uuid4()
 
 
 #________________________________________________________________
-class XMI:
-	@property
-	def doc(self):
-		return self.doc
+class XMI(object):
 
-	@property
-	def root(self):
-		return self.root
-
-	@property
-	def modelNS(self):
-		return self.modelNS
-
-	@property
-	def files(self):
-		return self.files
-
-	def __init__(self):
-		self.exporter = '$HeadURL: git@github.com:eddo888/Tools/xmi.py $'
-		self.version = '$Revision: 11983 $'
+	def __init__(self, name=None):
+		self.exporter = 'Enterprise Architect'
+		self.version = '2.5'
 		self.timestamp = '%Y-%m-%d %H:%M:%S'
 
 		self.classDiagramStyle = {
@@ -187,7 +177,7 @@ class XMI:
 
 		self.now = datetime.now()
 		self.makeDocument()
-		self.makeModel()
+		self.makeModel(name=name)
 		self.makeExtensions()
 		self.stereotypes = {}
 		return
@@ -215,16 +205,16 @@ class XMI:
 		meta.setProp('xmi.version', '1.4')
 		return
 
-	def makeModel(self):
+	def makeModel(self, name=None):
 		self.content = addElement(self.doc, 'XMI.content', self.root)
 		model = addElement(self.doc, 'UML:Model', self.content)
 		model.setProp('isAbstract', 'false')
 		model.setProp('isLeaf', 'false')
 		model.setProp('isRoot', 'true')
 		model.setProp('isSpecification', 'false')
-		model.setProp('name', self.now.strftime(self.timestamp))
+		model.setProp('name', name or self.now.strftime(self.timestamp))
 		model.setProp('visibility', 'public')
-		model.setProp('xmi.id', generateID())
+		model.setProp('xmi.id', generateID('Model',name))
 		self.modelNS = addElement(self.doc, 'UML:Namespace.ownedElement', model)
 		#self.rootClass = self.makeClass('EARootClass',self.modelNS)
 		#self.rootClass.parent.setProp('isRoot','true')
@@ -244,7 +234,7 @@ class XMI:
 		package.setProp('isSpecification', 'false')
 		package.setProp('name', name)
 		package.setProp('visibility', 'public')
-		package.setProp('xmi.id', generateID())
+		package.setProp('xmi.id', generateID('Package',name))
 		element = addElement(self.doc, 'UML:Namespace.ownedElement', package)
 		return element
 
@@ -255,14 +245,14 @@ class XMI:
 		return package
 
 	def makeCollaboration(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('Collaboration',name)
 		collaboration = addElement(self.doc, 'UML:Collaboration', parent)
 		collaboration.setProp('name', name)
 		collaboration.setProp('xmi.id', uid)
 		return addElement(self.doc, 'UML:Namespace.ownedElement', collaboration)
 
 	def makeRequirement(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('Requirement',name)
 		requirement = addElement(self.doc, 'UML:ClassifierRole', parent)
 		requirement.setProp('isAbstract', 'false')
 		requirement.setProp('isLeaf', 'false')
@@ -278,7 +268,7 @@ class XMI:
 		return addElement(self.doc, 'UML:Classifier.feature', requirement)
 
 	def makeActor(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('Actor',name)
 		actor = addElement(self.doc, 'UML:Actor', parent)
 		actor.setProp('isAbstract', 'false')
 		actor.setProp('isLeaf', 'false')
@@ -293,7 +283,7 @@ class XMI:
 		return addElement(self.doc, 'UML:Classifier.feature', actor)
 
 	def makeUseCase(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('UseCase',name)
 		useCase = addElement(self.doc, 'UML:UseCase', parent)
 		useCase.setProp('isAbstract', 'false')
 		useCase.setProp('isLeaf', 'false')
@@ -308,7 +298,7 @@ class XMI:
 		return addElement(self.doc, 'UML:Classifier.feature', useCase)
 
 	def makeComponent(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('Component',name)
 		system = addElement(self.doc, 'UML:Component', parent)
 		system.setProp('isAbstract', 'false')
 		system.setProp('isLeaf', 'false')
@@ -324,7 +314,7 @@ class XMI:
 		return addElement(self.doc, 'UML:Classifier.feature', system)
 
 	def makeClass(self, name, parent, uid=None):
-		if not uid: uid = generateID()
+		if not uid: uid = generateID('Class',name)
 		clasz = addElement(self.doc, 'UML:Class', parent)
 		clasz.setProp('isAbstract', 'false')
 		clasz.setProp('isLeaf', 'false')
@@ -384,7 +374,7 @@ class XMI:
 		if not tid:
 			return
 		parameterP = addElement(self.doc, 'UML:Parameter', parent)
-		parameterP.setProp('xmi.id', generateID())
+		parameterP.setProp('xmi.id', generateID('Parameter',name))
 		parameterP.setProp('name', name)
 		parameterP.setProp('kind', kind)
 		parameterT = addElement(self.doc, 'UML:Parameter.type', parameterP)
@@ -392,21 +382,14 @@ class XMI:
 		parameterC.setProp('xmi.idref', tid)
 		return
 
-	def makeAssociation(self,
-																					name,
-																					source,
-																					target,
-																					parent,
-																					sid=None,
-																					tid=None,
-																					array=False):
+	def makeAssociation(self, name, source, target, parent, sid=None, tid=None, array=False):
 		if source:
 			sid = source.parent.prop('xmi.id')
 		if target:
 			tid = target.parent.prop('xmi.id')
 		association = addElement(self.doc, 'UML:Association', parent)
 		association.setProp('name', name)
-		association.setProp('xmi.id', generateID())
+		association.setProp('xmi.id', generateID('Association',name))
 		association.setProp('visibility', 'public')
 		association.setProp('isAbstract', 'false')
 		association.setProp('isLeaf', 'false')
@@ -443,21 +426,14 @@ class XMI:
 		targetClass.setProp('xmi.idref', tid)
 		return association
 
-	def makeUsage(self,
-															name,
-															source,
-															target,
-															parent,
-															sid=None,
-															tid=None,
-															array=False):
+	def makeUsage(self, name, source, target, parent, sid=None, tid=None, array=False):
 		if source:
 			sid = source.parent.prop('xmi.id')
 		if target:
 			tid = target.parent.prop('xmi.id')
 		association = addElement(self.doc, 'UML:Association', parent)
 		association.setProp('name', name)
-		association.setProp('xmi.id', generateID())
+		association.setProp('xmi.id', generateID('Usage',name))
 		association.setProp('visibility', 'public')
 		association.setProp('isAbstract', 'false')
 		association.setProp('isLeaf', 'false')
@@ -484,7 +460,7 @@ class XMI:
 		cr = addElement(self.doc, 'UML:ClassifierRole', parent)
 		cr.setProp('name', name)
 		cr.setProp('visibility', 'public')
-		cr.setProp('xmi.id', generateID())
+		cr.setProp('xmi.id', generateID('Object',name))
 		#cr.setProp('base',self.rootClass.parent.prop('xmi.id'))
 		self.makeLocalTag('classifier', type.parent.prop('xmi.id'), cr)
 		self.makeLocalTag('classname', type.parent.prop('name'), cr)
@@ -522,7 +498,7 @@ class XMI:
 
 	def makeContentTag(self, name, value, parent):
 		tag = addElement(self.doc, 'UML:TaggedValue', self.content)
-		tag.setProp('xmi.id', generateID())
+		tag.setProp('xmi.id', generateID('ContentTag',name))
 		tag.setProp('tag', name)
 		tag.setProp('value', value)
 		tag.setProp('modelElement', parent.parent.prop('xmi.id'))
@@ -532,7 +508,7 @@ class XMI:
 		if not name in list(self.stereotypes.keys()):
 			ust = addElement(self.doc, 'UML:Stereotype', self.modelNS)
 			ust.setProp('name', name)
-			ust.setProp('xmi.id', generateID())
+			ust.setProp('xmi.id', generateID('Stereotype',name))
 			self.stereotypes[name] = ust
 		mes = addElement(self.doc, 'UML:ModelElement.stereotype', parent.parent)
 		stereotype = addElement(self.doc, 'UML:Stereotype', mes)
@@ -542,7 +518,7 @@ class XMI:
 	def makeActivityModel(self, name, parent):
 		model = addElement(self.doc, 'UML:ActivityGraph', parent)
 		model.setProp('name', name)
-		model.setProp('xmi.id', generateID())
+		model.setProp('xmi.id', generateID('ActivityModel',name))
 		return model
 
 	def makeTransition(self, source, target, parent, name=None):
@@ -551,7 +527,7 @@ class XMI:
 		if transitions is None:
 			transitions = addElement(self.doc, 'UML:StateMachine.transitions', parent)
 		transition = addElement(self.doc, 'UML:Transition', transitions)
-		transition.setProp('xmi.id', generateID())
+		transition.setProp('xmi.id', generateID('Transition',name))
 		if name != None:
 			transition.setProp('name', name)
 
@@ -572,7 +548,7 @@ class XMI:
 			partitions = addElement(self.doc, 'UML:ActivityGraph.partition', parent)
 		partition = addElement(self.doc, 'UML:Partition', partitions)
 		partition.setProp('name', name)
-		partition.setProp('xmi.id', generateID())
+		partition.setProp('xmi.id', generateID('ActivitySwimlane',name))
 		return partition
 
 	def addActivityToLane(self, state, parent):
@@ -638,48 +614,42 @@ class XMI:
 		diagram = self.makeDiagram(name, parent)
 		diagram.parent.setProp('diagramType', 'ComponentDiagram')
 		self.makeLocalTag('type', 'Logical', diagram.parent)
-		self.makeLocalTag('EAStyle',
-																				self.makeStyle(self.classDiagramStyle), diagram.parent)
+		self.makeLocalTag('EAStyle', self.makeStyle(self.classDiagramStyle), diagram.parent)
 		return diagram
 
 	def makeClassDiagram(self, name, parent):
 		diagram = self.makeDiagram(name, parent)
 		diagram.parent.setProp('diagramType', 'ClassDiagram')
 		self.makeLocalTag('type', 'Logical', diagram.parent)
-		self.makeLocalTag('EAStyle',
-																				self.makeStyle(self.classDiagramStyle), diagram.parent)
+		self.makeLocalTag('EAStyle', self.makeStyle(self.classDiagramStyle), diagram.parent)
 		return diagram
 
 	def makeActivityDiagram(self, name, parent):
 		diagram = self.makeDiagram(name, parent)
 		diagram.parent.setProp('diagramType', 'ActivityDiagram')
 		self.makeLocalTag('type', 'Activity', diagram.parent)
-		self.makeLocalTag('EAStyle',
-																				self.makeStyle(self.activityDiagramStyle), diagram.parent)
+		self.makeLocalTag('EAStyle', self.makeStyle(self.activityDiagramStyle), diagram.parent)
 		return diagram
 
 	def makeRequirementsDiagram(self, name, parent):
 		diagram = self.makeDiagram(name, parent)
 		diagram.parent.setProp('diagramType', 'CustomDiagram')
 		self.makeLocalTag('type', 'Custom', diagram.parent)
-		self.makeLocalTag('EAStyle',
-																				self.makeStyle(self.requirementsDiagramStyle),
-																				diagram.parent)
+		self.makeLocalTag('EAStyle', self.makeStyle(self.requirementsDiagramStyle), diagram.parent)
 		return diagram
 
 	def makeUseCaseDiagram(self, name, parent):
 		diagram = self.makeDiagram(name, parent)
 		diagram.parent.setProp('diagramType', 'UseCaseDiagram')
 		self.makeLocalTag('type', 'Logical', diagram.parent)
-		self.makeLocalTag('EAStyle',
-																				self.makeStyle(self.classDiagramStyle), diagram.parent)
+		self.makeLocalTag('EAStyle', self.makeStyle(self.classDiagramStyle), diagram.parent)
 		return diagram
 
 	def makeDiagram(self, name, parent):
 		diagram = addElement(self.doc, 'UML:Diagram', self.content)
 		diagram.setProp('name', name)
 		diagram.setProp('toolName', 'Enterprise Architect 2.5')
-		diagram.setProp('xmi.id', generateID())
+		diagram.setProp('xmi.id', generateID('Diagram',name))
 		diagram.setProp('owner', parent.parent.prop('xmi.id'))
 		self.makeLocalTag('package', parent.parent.prop('xmi.id'), diagram)
 		elements = addElement(self.doc, 'UML:Diagram.element', diagram)
